--[[

$Revision: 15 $

(C) Copyright 2009 Bethink (bethink at naef dot com)
See LICENSE.txt for license terms.

]]


----------------------------------------------------------------------
-- Acquire libraries and declare add-on

local L = LibStub("AceLocale-3.0"):GetLocale("BossNotes", true)
BossNotes = LibStub("AceAddon-3.0"):NewAddon("BossNotes",
	"AceConsole-3.0",
	"AceEvent-3.0",
	"AceTimer-3.0",
	"AceSerializer-3.0",
	"AceComm-3.0")
	
----------------------------------------------------------------------
-- Constants

-- Chat options
local ACE_OPTS = {
	name = "Boss Notes",
	handler = BossNotes,
	type = "group",
	args = {
		notes = {
			name = "Notes",
			desc = "Shows (or hides) the Boss Notes panel.",
			order = 1,
			guiHidden = true,
			type = "execute",
			func = "ToggleBossNotesFrame"
		},
		options = {
			name = "Options",
			desc = "Shows the options panel.",
			order = 2,
			guiHidden = true,
			type = "execute",
			func = "ShowOptions",
		},
		gameVersions = {
			name = "Game Versions",
			desc = "Choose the game versions to show encounters from.",
			order = 3,
			type = "group",
			args = {
				classic = {
					name = "Classic",
					order = 1,
					arg = "gameVersionClassic",
					type = "toggle",
					get = "GetGlobal",
					set = "SetGameVersion"
				},
				tbc = {
					name = "The Burning Crusade",
					order = 2,
					arg = "gameVersionTbc",
					type = "toggle",
					get = "GetGlobal",
					set = "SetGameVersion"
				},
				wotlk = {
					name = "Wrath of the Lich King",
					order = 3,
					arg = "gameVersionWotlk",
					type = "toggle",
					get = "GetGlobal",
					set = "SetGameVersion"
				}
			}
		},
		syndication = {
			name = "Syndication",
			desc = "Choose the channels where to publish your boss notes.",
			order = 4,
			type = "group",
			args = {
				guild = {
					name = "Guild",
					order = 1,
					arg = "syndicationGuild",
					type = "toggle",
					get = "GetGlobal",
					set = "SetSyndication"
				},
				raid = {
					name = "Raid",
					order = 2,
					arg = "syndicationRaid",
					type = "toggle",
					get = "GetGlobal",
					set = "SetSyndication"
				}
			}
		}
	}
}

-- Database version
BOSS_NOTES_DATABASE_VERSION = 1

-- Bindings
BINDING_HEADER_BOSS_NOTES = L["BINDING_HEADER_BOSS_NOTES"]
BINDING_NAME_BOSS_NOTES = L["BINDING_NAME_BOSS_NOTES"]


----------------------------------------------------------------------
-- Core handlers

-- One-time initialization
function BossNotes:OnInitialize()
	-- Database
	self.db = LibStub("AceDB-3.0"):New("BossNotesDB")
	
	-- Defaults
	self.db:RegisterDefaults({
		global = {
			gameVersionClassic = false,
			gameVersionTbc = false,
			gameVersionWotlk = true,
			syndicationGuild = true,
			syndicationRaid = true
		},
		char = {
			notes = {
			}
		}
	})
	
	-- Database version
	self.db.global.databaseVersion = BOSS_NOTES_DATABASE_VERSION
	
	-- Subsystems
	self:InitializeComm()
	self:InitializeData()
	self:InitializeBossNotesFrame()
		
	-- Options
	LibStub("AceConfig-3.0"):RegisterOptionsTable("BossNotes", ACE_OPTS, L["SLASH_COMMANDS"])
	self.options = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("BossNotes", "Boss Notes")

	-- LDB
	self.dataObject = LibStub("LibDataBroker-1.1"):NewDataObject("BossNotes", { 
		type = "data source",
		text = "General",
		icon = "Interface\\Icons\\INV_Misc_Note_06.blp",
		OnClick = function (frame, button)
			if button == "RightButton" then
				self:ShowOptions()
			else
				self:ToggleBossNotesFrame()
			end
		end,
		OnTooltipShow = function (tooltip)
			self:OnTooltipShow(tooltip)
		end
	})
end

-- Handles (re-)enabling of the addon
function BossNotes:OnEnable ()
	-- Register
	self:RegisterMessage("BOSS_NOTES_SELECTION")
	
	-- Subsystems
	self:EnableComm()
	self:EnableData()
	self:EnableBossNotesFrame()
end

-- Handles disabling of the addon
function BossNotes:OnDisable ()
	-- Subsystems
	self:DisableBossNotesFrame()
	self:DisableData()
	self:DisableComm()
end


----------------------------------------------------------------------
-- Event handlers

-- Handles the BOSS_NOTES_SELECTION event
function BossNotes:BOSS_NOTES_SELECTION (message, raid, encounter)
	self.dataObject.text = self:GetContextText(raid, encounter)
end

-- Handles the LDB show tooltip request
function BossNotes:OnTooltipShow (tooltip)
	-- Get the selection
	local raid, encounter, player = self:GetSelection()
	
	-- Format the header line
	local color = RAID_CLASS_COLORS[player.class]
	tooltip:AddLine(string.format("|cFF%.2x%.2x%.2x%s|r on |cFFFFFFFF%s|r",
			color.r * 255, color.g * 255, color.b * 255,
			player.name, self:GetContextText(raid, encounter)))
	tooltip:AddLine(" ")
	
	-- Format the note
	if player.note then
		for line in gmatch(player.note, "[^\r\n]+") do
			local pos = 1
			line = string.gsub(line, "%s+()(%S+)()", function (start, word, stop)
				if stop - pos > 60 then
					pos = start
					return "\n" .. word
				end
			end)
			tooltip:AddLine(line)
		end
	else
		tooltip:AddLine("No note.")
	end
end


----------------------------------------------------------------------
-- Settings

-- Shows or hides the Boss Notes frame
function BossNotes:ToggleBossNotesFrame ()
	if BossNotesFrame:IsShown() then
		HideUIPanel(BossNotesFrame)
	else
		ShowUIPanel(BossNotesFrame)
	end
end

-- Shows the options panel
function BossNotes:ShowOptions ()
	InterfaceOptionsFrame_OpenToCategory(self.options)
end

-- Gets a global setting
function BossNotes:GetGlobal (info)
	return self.db.global[info.arg]
end

-- Sets a game version setting
function BossNotes:SetGameVersion (info, value)
	self.db.global[info.arg] = value
	self:UpdateRaids()
end

-- Sets a syndication setting
function BossNotes:SetSyndication (info, value)
	self.db.global[info.arg] = value
end


----------------------------------------------------------------------
-- Support methods

-- Returns a text representation of the current context
function BossNotes:GetContextText (raid, encounter)
	if encounter then
		return encounter.name
	elseif raid then
		return raid.name
	else
		return "General"
	end
end
